"""
This is a stub package that allows the compat import mechanism injected in
zymbit/__init__.py

The importer will check the system this is running on, e.g. arduino,
raspberrypi, etc. and import modules from that package
"""
