from zymbit.config import *

INIT_SCRIPT_PATH = '/opt/zymbit/run'
FILES_ROOT = '/usr/local/files/raspberrypi'
KNOWN_HOSTS = '/etc/ssh/ssh_known_hosts'
ZYMBIT_KNOWN_HOSTS = '/etc/ssh/zymbit_known_hosts'
